# Backend endpoints

login = '/api/rest-auth/login/'
dataset = '/api/dataset/'
dataset_upload = '/api/dataset/{}/upload/'
dataset_folder_content = '/api/user-dataset/{}/contents/{}/'
v1_datasets = '/api/v1/ui/datasets/'
v1_dataset_annotation_sets = '/api/v1/ui/datasets/{}/annotation-sets/'
v1_dataset_image_annotations = '/api/v1/ui/datasets/{}/images/{}/annotations/'
v1_dataset_images = '/api/v1/ui/datasets/{}/images/'
v1_export_annotations = '/api/v1/ui/annotation-sets/{}/export/?annotation-format={}'
v1_search = '/api/v1/ui/search/'



from .domain.annotation_set import AnnotationSet
from .domain.dataset import Dataset, Image
from .domain import task
from .sdk import SDK

_logs = None
_server = None


def __init__():
    import os
    from pathlib import Path
    from .config import parse_config

    REMO_HOME = os.getenv('REMO_HOME', str(Path.home().joinpath('.remo')))
    cfg_path = str(os.path.join(REMO_HOME, 'remo.json'))
    config = parse_config(cfg_path)

    if config:

        server_url = '{}:{}'.format(config.server, config.port)

        def terminate_server():
            global _logs
            global _server
            print('Terminating Remo server')
            if _server:
                _server.terminate()
                _logs.close()

        def launch_server(open_browser=True):
            import time
            import requests
            from .browser import browse

            version_endpoint = '{}/version'.format(server_url)
            try:
                resp = requests.get(version_endpoint)
                print('Remo server is running:', resp.json())
                if open_browser:
                    browse('{}:{}'.format(config.server, config.port))

                return True
            except:
                pass

            print('Launching Remo server ...')

            if not os.path.exists(config.conda_env):
                print("""
    WARNING: Provided conda environment does not exist at {}
    
    Failed to launch Remo server, please start it manually.
    
    To fix config issue, please run init or update for Remo server (inside Remo conda env), like: 
    python -m remo_app init
    or
    python -m remo_app update
    """.format(config.conda_env))
                return False

            global _logs
            global _server
            _logs = open(str(os.path.join(REMO_HOME, 'logs.txt')), 'w')

            path_args = ['python'] if os.name == 'nt' else ['bin', 'python']
            python_exe = os.path.join(config.conda_env, *path_args)

            import subprocess
            _server = subprocess.Popen(
                '{} -m remo_app'.format(python_exe),
                stdout=_logs, stderr=_logs,
                shell=True, universal_newlines=True,
            )

            time.sleep(5)
            retry = 5
            while retry > 0:
                print('Wait a bit... ', retry)
                try:
                    resp = requests.get(version_endpoint)
                    print('Remo server is running:', resp.json())
                    if open_browser:
                        browse(server_url)

                    return True
                except:
                    time.sleep(1)
                retry -= 1

            print('Failed to launch Remo server, please start it manually')
            return False

        launch_server(open_browser=False)

        sdk = SDK(server_url, config.user_email, config.user_password)

        # set access to public SDK methods
        import sys
        is_public_sdk_method = lambda name: not name.startswith('_') and callable(getattr(sdk, name))
        functions = filter(is_public_sdk_method, dir(sdk))
        for name in functions:
            setattr(sys.modules[__name__], name, getattr(sdk, name))

        setattr(sys.modules[__name__], 'launch_server', launch_server)
        setattr(sys.modules[__name__], 'terminate_server', terminate_server)

    else:
        print("""
        WARNING: Config not found in {}
        
        Please run init or update for Remo server (inside Remo conda env), like: 
        
        python -m remo_app init
        or
        python -m remo_app update
        
        """.format(cfg_path))


__init__()



import json
import os


class Config:
    __slots__ = ['port', 'server', 'user_name', 'user_email', 'user_password', 'conda_env']
    _default_port = 8000
    _default_server = 'http://localhost'
    _default_user_name = 'Admin User'
    _default_user_email = 'admin@remo.ai'
    _default_user_password = 'adminpass'

    def __init__(self, config):
        self.port = config.get('port', self._default_port)
        self.server = config.get('server', self._default_server)

        self.user_name = config.get('user_name', self._default_user_name)
        self.user_email = config.get('user_email', self._default_user_email)
        self.user_password = config.get('user_password', self._default_user_password)

        self.conda_env = config.get('conda_env')


def parse_config(config_path):
    if not os.path.exists(config_path):
        return None

    with open(config_path) as cfg_file:
        config = json.load(cfg_file)

    return Config(config)



class AnnotationSet:

    def __init__(self, sdk, **kwargs):
        self.sdk = sdk
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.task = kwargs.get('task')
        self.total_classes = kwargs.get('total_classes')
        self.released_at = kwargs.get('released_at')
        self.updated_at = kwargs.get('updated_at')
        self.total_images = kwargs.get('total_images')
        self.top3_classes = kwargs.get('top3_classes')

    def __str__(self):
        return "Annotation set {id} - '{name}', task: {task}, #classes: {total_classes}".format(
            id=self.id, name=self.name, task=self.task, total_classes=self.total_classes)

    def __repr__(self):
        return self.__str__()

    def get_annotations(self, annotation_format=None):
        """
        :param annotation_format: choose format from this list ['json', 'coco']
        :return: annotations
        """
        args = [self.id]
        if annotation_format:
            args.append(annotation_format)
        return self.sdk.export_annotations(*args)



from .api import API
from .domain.dataset import Dataset
from .domain.annotation_set import AnnotationSet
from .browser import browse
from .endpoints import frontend


class SDK:
    def __init__(self, server, email, password):
        self.api = API(server, email, password)

    # TODO: Add a default annotation set as a dataset created
    def create_dataset(self, name, local_files=[], paths_to_upload=[], urls=[], annotation_task=None):
        """ 
        Creates a new dataset in Remo and optionally populate it with images and annotation from local drive or URL

        Args:
           name: string, name of the Dataset

           local_files: list of files or directories.
                Function will scan for .png, .jpeg, .tiff and .jpg in the folders and sub-folders.

           paths_to_upload: list of files or directories.
            These files will be uploaded to the local disk.
              - files supported: image files, annotation files and archive files.
              - Annotation files: json, xml, csv. If annotation file is provided, you need to provide annotation task.
              - Archive files: zip, tar, gzip. These files are unzipped, and then we scan for images, annotations and other archives. Support for nested archive files, image and annotation files in the same format supported elsewhere

           urls: list of urls pointing to downloadable target, which should be an archive file.
              The function will download the target of the URL, scan for archive files, unpack them and the results will be scanned for images, annotations and other archives.

           annotation_task:
               object_detection = 'Object detection'. Supports Coco, Open Images, Pascal
               instance_segmentation = 'Instance segmentation'. Supports Coco
               image_classification = 'Image classification'. ImageNet


        Returns: remo Dataset
        """

        result = self.api.create_dataset(name)
        my_dataset = Dataset(self, **result)
        my_dataset.add_data(local_files, paths_to_upload, urls, annotation_task)
        my_dataset.initialise_images()

        return my_dataset

    # MC: Can annotation_task have a default value if dataset has only one annotation format?
    def add_data_to_dataset(self, dataset_id, local_files=[],
                            paths_to_upload=[], urls=[], annotation_task=None, folder_id=None):
        """
        Adds data to existing dataset

        Args:
            - dataset_id: id of the desired dataset to extend (integer)
            - local_files: list of files or directories.
                    Function will scan for .png, .jpeg, .tiff and .jpg in the folders and sub-folders.
            - paths_to_upload: list of files or directories.
                    These files will be uploaded to the local disk.

               files supported: image files, annotation files and archive files.

               Annotation files: json, xml, csv.
                    If annotation file is provided, you need to provide annotation task.

               Archive files: zip, tar, gzip.
                    These files are unzipped, and then we scan for images, annotations and other archives. Support for nested archive files, image and annotation files in the same format supported elsewhere

            - urls: list of urls pointing to downloadable target, which should be an archive file.
                    The function will download the target of the URL - then we scan for archive files, unpack them and proceed as per Archive file section.

            - annotation_task:
               object_detection = 'Object detection'. Supports Coco, Open Images, Pascal
               instance_segmentation = 'Instance segmentation'. Supports Coco
               image_classification = 'Image classification'. ImageNet
            - folder_id: if there is a folder in the targer remo id, and you want to add images to a specific folder, you can specify it here.
        """

        result = {}
        if len(local_files):
            if type(local_files) is not list:
                raise ValueError(
                    'Function parameter "paths_to_add" should be a list of file or directory paths, but instead is a ' + str(
                        type(local_files)))

            files_upload_result = self.api.upload_local_files(dataset_id, local_files, annotation_task, folder_id)
            result['files_link_result'] = files_upload_result

        if len(paths_to_upload):
            if type(paths_to_upload) is not list:
                raise ValueError(
                    'Function parameter "paths_to_upload" should be a list of file or directory paths, but instead is a ' + str(
                        type(paths_to_upload)))

            files_upload_result = self.api.bulk_upload_files(dataset_id=dataset_id,
                                                             files_to_upload=paths_to_upload,
                                                             annotation_task=annotation_task,
                                                             folder_id=folder_id)

            result['files_upload_result'] = files_upload_result

        if len(urls):
            if type(urls) is not list:
                raise ValueError(
                    'Function parameter "urls" should be a list of URLs, but instead is a ' + str(type(urls)))

            urls_upload_result = self.api.upload_urls(dataset_id=dataset_id,
                                                      urls=urls,
                                                      annotation_task=annotation_task,
                                                      folder_id=folder_id)

            print(urls_upload_result)
            result['urls_upload_result'] = urls_upload_result
        return result

    def list_datasets(self):
        """
        Lists the available datasets
        """
        resp = self.api.list_datasets()
        return [
            Dataset(self, id=dataset['id'], name=dataset['name'])
            for dataset in resp.get('results', [])
        ]

    def get_dataset(self, dataset_id):
        """
        Given a dataset id returns the dataset
        Args:
            dataset_id: int
        Returns: remo dataset
        """
        result = self.api.get_dataset(dataset_id)
        return Dataset(self, **result)

    def list_annotation_sets(self, dataset_id):
        """
        Lists the annotation sets
        Args:
            dataset_id: int
        Returns: list of annotations containing annotation set id-name, annotation task and num classes
        """
        resp = self.api.list_annotation_sets(dataset_id)
        return [
            AnnotationSet(self,
                          id=annotation_set['id'],
                          name=annotation_set['name'],
                          task=annotation_set['task']['name'],
                          total_classes=annotation_set['statistics']['total_classes'])
            for annotation_set in resp.get('results', [])
        ]

    # TODO: convert into a dataset function
    def get_annotations(self, annotation_set_id, annotation_format='json'):
        """
        Args:
            annotation_format: 'json' or 'coco', default = 'json'
        Returns: annotations, format: list of dicts
        """
        return self.api.get_annotations(annotation_set_id, annotation_format)

    def export_annotation_to_csv(self, annotation_set_id, output_file, annotation_task):
        """
        Takes annotations and saves as a .csv file  
        Args:
            annotation_set_id: int
            output_file: .csv path
            annotation_task:
               object_detection = 'Object detection'. Supports Coco, Open Images, Pascal
               instance_segmentation = 'Instance segmentation'. Supports Coco
               image_classification = 'Image classification'. ImageNet
        """
        return self.api.export_annotation_to_csv(annotation_set_id, output_file, annotation_task)

    def annotation_statistics(self, dataset_id):
        """
        Shows annotation statistics
        
        Args:
            - dataset_id: int

        Returns: annotation set id, name, num of images, num of classes, num of objects, top3 classes, release and update dates
        """

        resp = self.api.list_annotation_sets(dataset_id)
        return [
            "Annotation set {id} - '{name}',  #images: {total_images}, #classes: {total_classes}, #objects: {total_annotation_objects}, Top3 classes: {top3_classes}, Released: {released_at}, Updated: {updated_at} ".format(
                id=annotation_set['id'], name=annotation_set['name'],
                total_images=annotation_set['total_images'],
                total_classes=annotation_set['statistics']['total_classes'],
                total_annotation_objects=annotation_set['statistics']['total_annotation_objects'],
                top3_classes=[(i['name'], i['count']) for i in annotation_set['statistics']['top3_classes']],
                released_at=annotation_set.get('released_at'),
                updated_at=annotation_set['updated_at'])
            for annotation_set in resp.get('results', [])
        ]

    def list_dataset_images(self, dataset_id, folder_id=None, **kwargs):
        """
        Given a dataset id returns list of the dataset images
        
        Args:
            - dataset_id: the id of the dataset to query
            - folder_id: the id of the folder to query
        Returns: list of images with their names and ids
        """

        if folder_id is not None:
            result = self.api.list_dataset_contents_by_folder(dataset_id, folder_id, **kwargs)
        else:
            result = self.api.list_dataset_contents(dataset_id, **kwargs)
        images = []

        for entry in result.get('results', []):
            my_dict = {}
            my_dict['name'] = entry.get('name')
            my_dict['id'] = entry.get('id')
            images.append(my_dict)

        return images

    def get_images(self, dataset_id, image_id):
        """
        Get image file by dataset_id and image_id
        Args:
            dataset_id: int
            image_id: int

        Returns: image
        """
        return self.api.get_images(dataset_id, image_id)

    def search_images(self, class_name, annotation_task):
        """
        Search images by class and annotation task
        Args:
            class_name: string.
                Name of the class to filter dataset.
            annotation_task:
               object_detection = 'Object detection'. Supports Coco, Open Images, Pascal
               instance_segmentation = 'Instance segmentation'. Supports Coco
               image_classification = 'Image classification'. ImageNet
        Returns: image_id, dataset_id, name, annotations
        """
        return self.api.search_images(class_name, annotation_task)

    def search_class(self, class_name):
        """
        Returns images with given class
        WIP
        """
        return self.api.search_class(class_name)

    def view_search(self, cls=None, task=None):
        # TODO: view search by class and task
        """
        Opens browser in search page

        """
        self._view(frontend.search)

    def view_image(self, image_id, dataset_id):
        """
        Opens browser on the image view for giving image
        Args:
            image_id: int
            dataset_id: int
        Returns: Browse UI of the selected image
        """
        # TODO: find easier way to check if image belongs to dataset
        img_list = self.list_dataset_images(dataset_id)
        contain = False
        for img_dict in img_list:
            if image_id == img_dict['id']:
                contain = True
                self._view(frontend.image_view.format(image_id, dataset_id))
        if not contain:
            msg = 'Image ID: %s' % str(image_id) + ' not in dataset %s' % str(dataset_id)
            print(msg)

    def view_datasets(self):
        self._view(frontend.datasets)

    def view_dataset(self, id):
        self._view(frontend.datasets, id)

    def view_annotation_set(self, id):
        self._view(frontend.annotation, id)

    def view_annotation_stats(self, annotation_id):
        self._view(frontend.annotation_detail.format(annotation_id))

    def _view(self, url, *args, **kwargs):
        browse(self.api.url(url, *args, **kwargs))



from io import BytesIO


class Image:
    """
    TODO: Start using it
    WIP
    """
    id = None
    dataset = None
    path_to_image = None
    annotation_sets = []

    def __init__(self, **kwargs):
        self.id = kwargs.get('id')
        self.dataset = kwargs.get('dataset')
        self.path_to_image = kwargs.get('path')
        # print(id, dataset,path_to_image, '\n')


class Dataset:
    def __init__(self, sdk, **kwargs):
        self.sdk = sdk
        self.id = kwargs.get('id')
        self.name = kwargs.get('name')
        self.images = []
        self._annotation_sets = []
        self.default_annotation_set = None

    def __str__(self):
        return "Dataset {id} - '{name}'".format(id=self.id, name=self.name)

    def __repr__(self):
        return self.__str__()

    def annotation_sets(self):
        """
        Returns a list of the annotation sets within the dataset
        """

        return self.sdk.list_annotation_sets(self.id)

    def annotation_statistics(self):
        """
        Prints annotation statistics of the dataset
        TODO - for which annotation set?
        WIP
        
        """
        return self.sdk.annotation_statistics(self.id)

    def initialise_images(self):
        list_of_images = self.list_images()
        for i_image in list_of_images:
            my_image = Image(id=None, path=i_image, dataset=self.name)
            self.images.append(my_image)

    def list_images(self, folder_id=None, **kwargs):
        """
        Prints annotation statistics of the dataset
        TODO - for which annotation set?
        WIP
        
        """
        return self.sdk.list_dataset_images(self.id, folder_id=None, **kwargs)

    def add_data(self, local_files=[], paths_to_upload=[], urls=[], annotation_task=None, folder_id=None):
        """
            
        Adds data to the dataset
        
        Args:
            - local_files: list of files or directories. Function will scan for .png, .jpeg, .tiff and .jpg in the folders and sub-folders.
            - paths_to_upload: list of files or directories. These files will be uploaded to the local disk.

               files supported: image files, annotation files and archive files.

               Annotation files: json, xml, csv. If annotation file is provided, you need to provide annotation task.

               Archive files: zip, tar, gzip. These files are unzipped, and then we scan for images, annotations and other archives. Support for nested archive files, image and annotation files in the same format supported elsewhere

            - urls: list of urls pointing to downloadable target, which should be an archive file. The function will download the target of the URL - then we scan for archive files, unpack them and proceed as per Archive file section.

            - annotation_task:
               object_detection = 'Object detection'. Supports Coco, Open Images, Pascal
               instance_segmentation = 'Instance segmentation'. Supports Coco
               image_classification = 'Image classification'. ImageNet
            - folder_id: if there is a folder in the targer remo id, and you want to add images to a specific folder, you can specify it here.
        """

        return self.sdk.add_data_to_dataset(dataset_id=self.id,
                                            local_files=local_files,
                                            paths_to_upload=paths_to_upload,
                                            urls=urls,
                                            annotation_task=annotation_task,
                                            folder_id=folder_id)

    def fetch(self):
        dataset = self.sdk.get_dataset(self.id)
        self.__dict__.update(dataset.__dict__)

    def view(self):
        self.sdk.view_dataset(self.id)

    def view_annotation(self, annotation_set_id):
        # TODO: select by annotation task
        self.sdk.view_annotation_set(annotation_set_id)

    def view_annotation_statistics(self, ann_id):
        self.sdk.view_annotation_stats(ann_id)

    def get_images(self, image_id, cls=None, tag=None):
        # TODO: add class and tags & multiple images
        r = self.sdk.get_images(self.id, image_id)
        return BytesIO(r.content)

    def view_image(self, image_id, cls=None, tag=None):
        return self.sdk.view_image(image_id, self.id)

    def view_search(self, **kwargs):
        pass

    def view_objects(self, cls, tag):
        pass



import http
import os
import csv
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime, timedelta
import filetype
import requests
import urllib.parse

from .domain import task
from .endpoints import backend
from .utils import FileResolver


class UploadStatus:

    def __init__(self, total_count):
        self.total_count = total_count
        self.current_count = 0
        self.start = datetime.now()
        self.reported_progress = 0

    def update(self, count):
        self.current_count += count

    def progress(self):
        percentage = int(self.current_count / self.total_count * 100)
        if percentage > self.reported_progress:
            elapsed = (datetime.now() - self.start).seconds + 1e-3
            avg_speed = self.current_count / elapsed
            eta = timedelta(seconds=(self.total_count - self.current_count) / avg_speed)
            print(
                'Progress {}% - {}/{} - elapsed {} - speed: {} img / s, ETA: {}'.format(percentage, self.current_count,
                                                                                        self.total_count,
                                                                                        timedelta(seconds=elapsed),
                                                                                        "%.2f" % avg_speed, eta))
        self.reported_progress = percentage


class BaseAPI:
    def __init__(self, server, email, password):
        self.server = server
        self.token = None
        self._login(email, password)

    def _login(self, email, password):
        try:
            resp = requests.post(self.url(backend.login),
                                 data={"password": password, "email": email})
        except requests.exceptions.ConnectionError:
            print('ERROR: Failed connect to server')
            return

        if resp.status_code != http.HTTPStatus.OK:
            raise Exception(resp.json())

        self.token = resp.json().get('key')

    def _is_authenticated(self):
        return self.token is not None

    def _auth_header(self):
        if not self._is_authenticated():
            raise Exception('Not authenticated')
        return {'Authorization': 'Token {}'.format(self.token)}

    def url(self, endpoint, *args, **kwargs):
        return self._build_url(self.server, endpoint, *args, **kwargs)

    def post(self, *args, **kwargs):
        return requests.post(*args, headers=self._auth_header(), **kwargs)

    def get(self, *args, **kwargs):
        return requests.get(*args, headers=self._auth_header(), **kwargs)

    @staticmethod
    def _build_url(*args, **kwargs):
        """
        Builds full url from inputs.
        Additional param `tail_slash` specifies tailed slash

        :param args: typically server and endpoint
        :param kwargs: additional query parameters
        :return: full url
        """
        args = list(filter(lambda arg: arg is not None, args))
        tail_slash = kwargs.pop('tail_slash', (str(args[-1])[-1] == '/'))
        url = '/'.join(map(lambda x: str(x).strip('/'), args))

        params = []
        for key, val in kwargs.items():
            if val:
                params.append('{}={}'.format(key, val))

        if len(params):
            joined_params = "&".join(params)
            separator = '&' if '?' in url else '/?'
            url = "{}{}{}".format(url, separator, urllib.parse.quote(joined_params))
        elif '?' not in url and tail_slash:
            url += '/'

        return url


class API(BaseAPI):

    def create_dataset(self, name):
        return self.post(self.url(backend.dataset), json={"name": name}).json()

    def upload_file(self, dataset_id, path, annotation_task=None, folder_id=None):
        name = os.path.basename(path)
        files = {'files': (name, open(path, 'rb'), filetype.guess_mime(path))}
        data = {}
        if annotation_task:
            data['annotation_task'] = annotation_task

        url = self.url(backend.dataset_upload.format(dataset_id), folder_id=folder_id)
        return self.post(url, files=files, data=data).json()

    # TODO: fix progress to include both local files and uploads
    def upload_files(self, dataset_id, files_to_upload=[], annotation_task=None, folder_id=None, status=None):
        files = [('files', (os.path.basename(path), open(path, 'rb'), filetype.guess_mime(path))) for path in
                 files_to_upload]
        data = {}
        if annotation_task:
            data['annotation_task'] = annotation_task

        url = self.url(backend.dataset_upload.format(dataset_id), folder_id=folder_id)

        r = self.post(url, files=files, data=data)

        if r.status_code != http.HTTPStatus.OK:
            print('Possible Error - Response:', r.text, 'files:', files_to_upload)

        status.update(len(files))
        status.progress()
        return r.json()

    # TODO: fix progress to include both local files and uploads
    def bulk_upload_files(self, dataset_id, files_to_upload, annotation_task=None, folder_id=None):

        # files to upload
        files = FileResolver(files_to_upload, annotation_task is not None).resolve()
        groups = self.split_files_by_size(files)
        status = UploadStatus(len(files))
        with ThreadPoolExecutor(1) as ex:
            res = ex.map(lambda bulk: self.upload_files(dataset_id, bulk, annotation_task, folder_id, status),
                         groups)

        results = res
        return results

    def chunks(self, my_list, chunk_size=2000):
        groups = []
        """Yield successive n-sized chunks from l."""
        for i in range(0, len(my_list), chunk_size):
            groups.append(my_list[i:i + chunk_size])

        return groups

    def split_files_by_size(self, files):
        groups = []
        bulk = []
        total_size = 0
        bulk_size = 8 * 1024 * 1024
        for path in files:
            total_size += os.path.getsize(path)
            bulk.append(path)
            if total_size >= bulk_size:
                groups.append(bulk)
                bulk = []
                total_size = 0

        if len(bulk):
            groups.append(bulk)
        return groups

    def upload_local_files(self, dataset_id, local_files, annotation_task=None, folder_id=None):
        payload = {"local_files": local_files}
        if annotation_task:
            payload['annotation_task'] = annotation_task
        if folder_id:
            payload['folder_id'] = folder_id

        url = self.url(backend.dataset_upload.format(dataset_id))
        return self.post(url, json=payload).json()

    def upload_urls(self, dataset_id, urls, annotation_task=None, folder_id=None):
        payload = {"urls": urls}
        if annotation_task:
            payload['annotation_task'] = annotation_task
        if folder_id:
            payload['folder_id'] = folder_id

        url = self.url(backend.dataset_upload.format(dataset_id))
        return self.post(url, json=payload).json()

    def list_datasets(self):
        url = self.url(backend.v1_datasets)
        return self.get(url).json()

    def list_dataset_contents(self, dataset_id, **kwargs):
        url = self.url(backend.v1_dataset_images.format(dataset_id), **kwargs)
        return self.get(url).json()

    def list_dataset_contents_by_folder(self, dataset_id, folder_id, **kwargs):
        url = self.url(backend.dataset_folder_content.format(dataset_id, folder_id), **kwargs)
        return self.get(url).json()

    def get_dataset(self, id):
        url = self.url(backend.dataset, id, tail_slash=True)
        print('url', url)
        r = self.get(url)
        print(r.content)
        return r.json()

    def list_annotation_sets(self, dataset_id):
        url = self.url(backend.v1_dataset_annotation_sets.format(dataset_id))
        return self.get(url).json()

    def get_annotations(self, annotation_set_id, annotation_format='json'):
        """
        Args:
            annotation_format: can be one of ['json', 'coco'], default='json'
        Returns: annotations
        """
        url = self.url(backend.v1_export_annotations.format(annotation_set_id, annotation_format))
        return self.get(url).json()

    # TODO: seems like this func should go to sdk
    def export_annotation_to_csv(self, annotation_set_id, output_file, annotation_task):
        annotation = self.get_annotations(annotation_set_id, annotation_format='json')
        output = open(output_file, 'w', newline='')
        f = csv.writer(output)
        if annotation_task == task.object_detection:
            header = ['file_name', 'class', 'xmin', 'ymin', 'xmax', 'ymax']
            f.writerow(header)
            for item in annotation:
                annotations = item['annotations']
                for annotation in annotations:
                    classes = annotation['classes']
                    for cls in classes:
                        f.writerow([item['file_name'], cls] + list(annotation['bbox'].values()))
        elif annotation_task == task.instance_segmentation:
            header = ['file_name', 'class', 'coordinates']
            f.writerow(header)
            for item in annotation:
                annotations = item['annotations']
                for annotation in annotations:
                    classes = annotation['classes']
                    for cls in classes:
                        segments = ' '.join(str(v) for d in annotation['segments'] for k, v in d.items())
                        f.writerow([item['file_name'], cls, segments])
        else:
            # TODO: classification task 
            # MC: currently there is a bug in the ui
            output.close()
            raise NotImplementedError
        output.close()

    def get_images(self, dataset_id, image_id):
        url = self.url(backend.v1_dataset_image_annotations.format(dataset_id, image_id))
        content = self.get(url).json()
        image_url = self.url(content['image'])
        return self.get(image_url)

    def search_images(self, class_name, annotation_task, num_data=None):
        url = self.url(backend.v1_search, classes=class_name, tasks=task)
        results = self.get(url).json()['results']
        return results



from setuptools import setup, find_packages

setup(
    name='remo-sdk',
    version='0.0.7',
    author='Rediscovery',
    author_email='hello@remo.ai',
    packages=find_packages(exclude=['backup', 'example']),
    url='https://github.com/rediscovery-io/remo-python',
    license='LICENSE.txt',
    description='Remo sdk',
    install_requires=[
        'filetype>=1.0.5',
        'requests>=2.21.0',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.5',
)
